/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Christian Meyer
   co-programmer: ...
*/

#include "load_param.h"

#include "list.h"
#include "base_object.h"

#include <stdarg.h>


BaseLoadParam::BaseLoadParam(BaseObject* object, const char* paramName, int paramCount, ...)
{
  if (LoadClassDescription::parametersDescription)
    {
      // locating the class
      this->classDesc = LoadClassDescription::addClass(object->getClassName());
      this->paramDesc = this->classDesc->addParam(paramName);

      this->paramDesc->types = new char*[paramCount];
      this->paramDesc->paramCount = paramCount;

      va_list types;
      va_start (types, paramCount);
      for(int i = 0; i < paramCount; i++)
	{
	  const char* tmpTypeName = va_arg (types, const char*);
	  this->paramDesc->types[i] = new char[strlen(tmpTypeName)+1];
	  strcpy(this->paramDesc->types[i], tmpTypeName);
	}
      va_end(types);  

      int argCount = 0;
    }
}


void BaseLoadParam::describe(const char* descriptionText)
{
  if (this->paramDesc)
    {
      
    }

}


LoadParamDescription::LoadParamDescription(const char* paramName)
{
  this->types = NULL;
  this->paramName = new char[strlen(paramName)+1];
  strcpy(this->paramName, paramName);
}

LoadParamDescription::~LoadParamDescription(void)
{
  for(int i = 0; i < this->paramCount; i++)
    {
      delete this->types[i];
    }
  delete []this->types;
  delete []this->paramName;
}


tList<LoadClassDescription>* LoadClassDescription::classList = new tList<LoadClassDescription>;

/**
   \brief if the description of Parameters should be executed
*/
bool LoadClassDescription::parametersDescription = true;

LoadClassDescription::LoadClassDescription(const char* className)
{
  this->className = new char[strlen(className)+1];
  strcpy(this->className, className);

  classList->add(this);

  this->paramList = new tList<LoadParamDescription>;
}

LoadClassDescription::~LoadClassDescription(void)
{
  delete []this->className;

  tIterator<LoadParamDescription>* iterator = this->paramList->getIterator();
  LoadParamDescription* enumParamDesc = iterator->nextElement();
  while (enumParamDesc)
    {
      delete enumParamDesc;
      enumParamDesc = iterator->nextElement();
    }
  delete iterator;

}


LoadClassDescription* LoadClassDescription::addClass(const char* className)
{
  tIterator<LoadClassDescription>* iterator = LoadClassDescription::classList->getIterator();
  LoadClassDescription* enumClassDesc = iterator->nextElement();
  while (enumClassDesc)
    {
      if (!strcmp(enumClassDesc->className, className))
	{
	  delete iterator;
	  return enumClassDesc;
	}
      enumClassDesc = iterator->nextElement();
    }
  delete iterator;

  return new LoadClassDescription(className);
}


LoadParamDescription* LoadClassDescription::addParam(const char* paramName)
{
  tIterator<LoadParamDescription>* iterator = this->paramList->getIterator();
  LoadParamDescription* enumParamDesc = iterator->nextElement();
  while (enumParamDesc)
    {
      if (!strcmp(enumParamDesc->paramName, paramName))
	{
	  delete iterator;
	  this->paramList->add(enumParamDesc);
	  return enumParamDesc;
	}
      enumParamDesc = iterator->nextElement();
    }
  delete iterator;

  this->paramList->add(new LoadParamDescription(paramName));
  return paramList->lastElement();
}
